package com.juxin.client.websocket.enums;

/**
 * 案件流程枚举
 */
public enum CaseFlowEnum {

    BANK_CARD_FUNDS_COMMIT("银行卡资金查询提交", 1, "rpa_bank_card_funds_case", "rpa_bank_card_funds_process"),
    THIRD_PARTY_ALL_ACCOUNT_COMMIT("第三方全账户主体查询提交", 2, "rpa_third_party_all_account", "rpa_third_party_all_account_group"),
    SERIAL_NUMBER_COMMIT("流水号资金查询提交", 3, "rpa_serial_number", "rpa_serial_number_group"),
    THIRD_FUNDS_COMMIT("第三方资金查询提交", 4, "rpa_third_funds_commit_case", "rpa_third_funds_commit_group"),
    BANK_CARD_FUNDS_SPLIT_RESULT("银行卡资金结果查询参数拆分", 5, "rpa_bank_card_funds_split", ""),
    THIRD_FUNDS_SPLIT_RESULT("第三方资金结果查询参数拆分", 6, "rpa_third_funds_split", ""),
    THIRD_PARTY_ALL_ACCOUNT_SPLIT_RESULT("第三方全账户资金结果查询参数拆分", 7, "rpa_third_party_all_account_split", ""),
    SERIAL_NUMBER_SPLIT_RESULT("流水号资金结果查询参数拆分", 8, "rpa_serial_number_split", "");
    /**
     * 流程名称
     */
    private String processName;
    /**
     * 流程id
     */
    private Integer processId;
    /**
     * 主表名称
     */
    private String mainTable;
    /**
     * 分组表名称
     */
    private String groupTable;

    CaseFlowEnum(String processName, Integer processId, String mainTable, String groupTable) {
        this.processName = processName;
        this.processId = processId;
        this.mainTable = mainTable;
        this.groupTable = groupTable;
    }

    public String getProcessName() {
        return processName;
    }

    public void setProcessName(String processName) {
        this.processName = processName;
    }

    public Integer getProcessId() {
        return processId;
    }

    public void setProcessId(Integer processId) {
        this.processId = processId;
    }

    public String getMainTable() {
        return mainTable;
    }

    public void setMainTable(String mainTable) {
        this.mainTable = mainTable;
    }

    public String getGroupTable() {
        return groupTable;
    }

    public void setGroupTable(String groupTable) {
        this.groupTable = groupTable;
    }

    /**
     * 根据流程名获取流程id
     *
     * @param processName
     * @return
     */
    public static Integer getProcessId(String processName) {
        for (CaseFlowEnum flowEnum : CaseFlowEnum.values()) {
            if (processName.equals(flowEnum.getProcessName())) {
                return flowEnum.getProcessId();
            }
        }
        return 0;
    }

    /**
     * 根据流程id获取流程名
     *
     * @param processId
     * @return
     */
    public static String getProcessName(Integer processId) {
        for (CaseFlowEnum flowEnum : CaseFlowEnum.values()) {
            if (processId == flowEnum.getProcessId()) {
                return flowEnum.getProcessName();
            }
        }
        return null;
    }

    public static CaseFlowEnum getCaseFlowEnum(Integer processId) {
        for (CaseFlowEnum flowEnum : CaseFlowEnum.values()) {
            if (processId == flowEnum.getProcessId()) {
                return flowEnum;
            }
        }
        return null;
    }

    public static CaseFlowEnum getCaseFlowEnumByName(String processName) {
        for (CaseFlowEnum flowEnum : CaseFlowEnum.values()) {
            if (processName.equals(flowEnum.getProcessName())) {
                return flowEnum;
            }
        }
        return null;
    }

    /**
     * 校验当前流程是否为提交查询流程
     *
     * @param processId
     * @return
     */
    public static Boolean judgeProcessIsCommit(Integer processId) {
        if (BANK_CARD_FUNDS_COMMIT.getProcessId() == processId ||
                THIRD_PARTY_ALL_ACCOUNT_COMMIT.getProcessId() == processId ||
                SERIAL_NUMBER_COMMIT.getProcessId() == processId ||
                THIRD_FUNDS_COMMIT.getProcessId() == processId) {
            return true;
        }
        return false;
    }

    /**
     * 流程类型判断 1查询流程  2查询结果流程
     * @return
     */
    public static int flowType(CaseFlowEnum flowEnum) {
      if(flowEnum == BANK_CARD_FUNDS_COMMIT || flowEnum == THIRD_PARTY_ALL_ACCOUNT_COMMIT
      || flowEnum == SERIAL_NUMBER_COMMIT || flowEnum == THIRD_FUNDS_COMMIT) {
           return 1;
      }else {
          return 2;
      }
    }
}
