package com.juxin.client.websocket;

import com.juxin.client.websocket.ppojo.WebsocketInfo;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import javax.websocket.Session;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatThrownBy;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
class FindTaskTest {

    @Mock
    private WebSocketClient mockClient;

    @InjectMocks
    private FindTask findTaskUnderTest;

    @Test
    void testInit() {
        // Setup
        // Run the test
        findTaskUnderTest.init();

        // Verify the results
        verify(mockClient).setUserSession(any(Session.class));
    }

    @Test
    void testHeart() throws Exception {
        // Setup
        // Configure WebSocketClient.getUserSession(...).
        final Session mockSession = mock(Session.class);
        when(mockClient.getUserSession()).thenReturn(mockSession);

        // Run the test
        findTaskUnderTest.heart();

        // Verify the results
        verify(mockSession).close();
        verify(mockClient).sendMessage(
                "{ \"infoType\": \"heart\", \"postType\": \"python\", \"methodType\": \"heart\", \"reqJson\": { \"receiveJobStatus\": \"1\", \"msg\": \"ping\" } }");
    }

    @Test
    void testHeart_WebSocketClientGetUserSessionReturnsNull() throws Exception {
        // Setup
        when(mockClient.getUserSession()).thenReturn(null);

        // Run the test
        findTaskUnderTest.heart();

        // Verify the results
        verify(mockClient).setUserSession(any(Session.class));
        verify(mockClient).sendMessage(
                "{ \"infoType\": \"heart\", \"postType\": \"python\", \"methodType\": \"heart\", \"reqJson\": { \"receiveJobStatus\": \"1\", \"msg\": \"ping\" } }");
    }

    @Test
    void testHeart_WebSocketClientSendMessageThrowsInterruptedException() throws Exception {
        // Setup
        // Configure WebSocketClient.getUserSession(...).
        final Session mockSession = mock(Session.class);
        when(mockClient.getUserSession()).thenReturn(mockSession);

        doThrow(InterruptedException.class).when(mockClient).sendMessage(
                "{ \"infoType\": \"heart\", \"postType\": \"python\", \"methodType\": \"heart\", \"reqJson\": { \"receiveJobStatus\": \"1\", \"msg\": \"ping\" } }");

        // Run the test
        findTaskUnderTest.heart();

        // Verify the results
        verify(mockSession).close();
    }

    @Test
    void testFindResultTask() throws Exception {
        // Setup
        // Configure WebSocketClient.getUserSession(...).
        final Session mockSession = mock(Session.class);
        when(mockClient.getUserSession()).thenReturn(mockSession);

        // Run the test
        findTaskUnderTest.findResultTask();

        // Verify the results
        verify(mockSession).close();
        verify(mockClient).sendMessage(
                "{ \"infoType\": \"getResultQueueJob\", \"postType\": \"python\", \"methodType\": \"getResultQueueJob\" }");
    }

    @Test
    void testFindResultTask_WebSocketClientGetUserSessionReturnsNull() {
        // Setup
        when(mockClient.getUserSession()).thenReturn(null);

        // Run the test
        findTaskUnderTest.findResultTask();

        // Verify the results
    }

    @Test
    void testFindResultTask_WebSocketClientSendMessageThrowsInterruptedException() throws Exception {
        // Setup
        // Configure WebSocketClient.getUserSession(...).
        final Session mockSession = mock(Session.class);
        when(mockClient.getUserSession()).thenReturn(mockSession);

        doThrow(InterruptedException.class).when(mockClient).sendMessage(
                "{ \"infoType\": \"getResultQueueJob\", \"postType\": \"python\", \"methodType\": \"getResultQueueJob\" }");

        // Run the test
        findTaskUnderTest.findResultTask();

        // Verify the results
        verify(mockSession).close();
    }

    @Test
    void testFindTask() throws Exception {
        // Setup
        // Configure WebSocketClient.getUserSession(...).
        final Session mockSession = mock(Session.class);
        when(mockClient.getUserSession()).thenReturn(mockSession);

        // Run the test
        findTaskUnderTest.findTask();

        // Verify the results
        verify(mockSession).close();
        verify(mockClient).sendMessage(
                "{ \"infoType\": \"getQueueJob\", \"postType\": \"python\", \"methodType\": \"getQueueJob\" }");
    }

    @Test
    void testFindTask_WebSocketClientGetUserSessionReturnsNull() {
        // Setup
        when(mockClient.getUserSession()).thenReturn(null);

        // Run the test
        findTaskUnderTest.findTask();

        // Verify the results
    }

    @Test
    void testFindTask_WebSocketClientSendMessageThrowsInterruptedException() throws Exception {
        // Setup
        // Configure WebSocketClient.getUserSession(...).
        final Session mockSession = mock(Session.class);
        when(mockClient.getUserSession()).thenReturn(mockSession);

        doThrow(InterruptedException.class).when(mockClient).sendMessage(
                "{ \"infoType\": \"getQueueJob\", \"postType\": \"python\", \"methodType\": \"getQueueJob\" }");

        // Run the test
        findTaskUnderTest.findTask();

        // Verify the results
        verify(mockSession).close();
    }

    @Test
    void testDealTask() throws Exception {
        // Setup
        // Run the test
        findTaskUnderTest.dealTask("message");

        // Verify the results
        verify(mockClient).sendMessage("");
    }

    @Test
    void testDealTask_WebSocketClientThrowsInterruptedException() throws Exception {
        // Setup
        doThrow(InterruptedException.class).when(mockClient).sendMessage("");

        // Run the test
        assertThatThrownBy(() -> findTaskUnderTest.dealTask("message")).isInstanceOf(RuntimeException.class);
    }

    @Test
    void testCommitTask() {
        // Setup
        final WebsocketInfo bean = WebsocketInfo.builder()
                .infoType("infoType")
                .methodType("methodType")
                .reqJson("reqJson")
                .build();

        // Run the test
        final String result = findTaskUnderTest.commitTask(bean);

        // Verify the results
        assertThat(result).isEqualTo("");
    }

    @Test
    void testBankCommitTask() {
        assertThat(findTaskUnderTest.bankCommitTask("reqJson")).isEqualTo("result");
    }

    @Test
    void testThirdCommitTask() {
        assertThat(findTaskUnderTest.thirdCommitTask("reqJson")).isEqualTo("result");
    }

    @Test
    void testSerialCommitTask() {
        assertThat(findTaskUnderTest.serialCommitTask("reqJson")).isEqualTo("result");
    }

    @Test
    void testAllAccountCommitTask() {
        assertThat(findTaskUnderTest.allAccountCommitTask("reqJson")).isEqualTo("result");
    }

    @Test
    void testSearchTask() {
        // Setup
        final WebsocketInfo bean = WebsocketInfo.builder()
                .infoType("infoType")
                .methodType("methodType")
                .reqJson("reqJson")
                .build();

        // Run the test
        final String result = findTaskUnderTest.searchTask(bean);

        // Verify the results
        assertThat(result).isEqualTo("");
    }

    @Test
    void testBankResultTask() {
        assertThat(findTaskUnderTest.bankResultTask("reqJson")).isEqualTo("result");
    }

    @Test
    void testThirdResultTask() {
        assertThat(findTaskUnderTest.thirdResultTask("reqJson")).isEqualTo("result");
    }

    @Test
    void testSerialResultTask() {
        assertThat(findTaskUnderTest.serialResultTask("reqJson")).isEqualTo("result");
    }

    @Test
    void testAllAccountResultTask() {
        assertThat(findTaskUnderTest.allAccountResultTask("reqJson")).isEqualTo("result");
    }
}
